/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file dune2.c
 * \author Janusz Dziemidowicz
 * \brief Dune 2 PAK format support header file.
 */

#include "dune2.h"

/*!
 * \brief Read \c dune2_entry from file.
 *
 * Tries to read one \c dune2_entry from given file. Because those
 * entries have variable size \c filename field of \c dune2_entry will
 * be malloc'ed. If field \c offset contains 0 then it is the last
 * entry (no filename will be read or malloc'ed).
 * \param fd descriptor of file to read from.
 * \param entry pointer to \c dune2_entry structure which should be
 * filled.
 * \return 0 if \c one entry was read successfully, -1 on errors, 1 if
 * it was unable to read one entry.
 */
static int read_dune2_entry(int fd, struct dune2_entry *entry)
{
	char buffer[4];
	char *str;
	int ret;

	ret = read(fd, buffer, sizeof(buffer));

	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	entry->offset = LE_32(buffer);

	// Offset of value 0 indicates last (empty) dune2_entry
	if (!entry->offset) {
		entry->filename = NULL;
		return 0;
	}

	str = read_string(fd);
	if (!str)
		return 1;

	entry->filename = strdup(str);

	return 0;
}

// **********************************************************************
// fusepak operations
// **********************************************************************

/*!
 * \brief Initialization/detection function.
 */
static struct tree *dune2_init(const int fd)
{
	struct dune2_entry entry, prev;
	off_t size;
	struct tree *tree;

	size = lseek(fd, 0, SEEK_END);
	lseek(fd, 0, SEEK_SET);

	tree = tree_empty();

	// Read first file entry
	if (read_dune2_entry(fd, &prev)) {
		fprintf(stderr,
			"unable to read file entry from Dune 2 PAK file\n");
		tree_free(tree);
		return NULL;
	}
	if (!prev.offset)
		return tree;

	for (;;) {
		if (read_dune2_entry(fd, &entry)) {
			fprintf(stderr,
				"unable to read file entry from Dune 2 PAK file\n");
			tree_free(tree);
			return NULL;
		}
		// End of directory
		if (!entry.offset)
			break;

		if (entry.offset > size) {
			fprintf(stderr, "malformed Dune 2 PAK file\n");
			tree_free(tree);
			return NULL;
		}

		tree_insert(tree, prev.filename, strlen(prev.filename),
			    prev.offset, entry.offset - prev.offset);
		free(prev.filename);
		prev = entry;
	}

	// Last file
	tree_insert(tree, prev.filename, strlen(prev.filename),
		    prev.offset, size - prev.offset);
	free(prev.filename);

	return tree;
}

struct fusepak_format fusepak_dune2 = {
	.format_name = "DUNE2",
	.init = dune2_init,
};
