/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file fusepak.c
 * \author Janusz Dziemidowicz
 * \brief Framework for adding new formats.
 */

#include "fusepak.h"

/*!
 * \brief Table of supported file formats.
 * 
 * This table contains \c fusepak_format structures, one for each
 * supported format. Each new format should be added here.
 */
static struct fusepak_format *fusepak_formats[] = {
	&fusepak_ksgrp,
	&fusepak_pack,
	&fusepak_wad,
	&fusepak_wad2,
	&fusepak_lgp,
	&fusepak_dune2,
};

//! GRAF descriptor.
static int fd;

//! Current directory tree.
static struct tree *tree;

//! Mutex used to serialized calls to \c read().
static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

// **********************************************************************
// FUSE operations
// Here comes FUSE operations, please consult fuse.h for description of
// each operation.
// **********************************************************************

/*!
 * \brief Get file attributes.
 */
static int fusepak_getattr(const char *path, struct stat *stbuf)
{
	return tree_getattr(path, stbuf, tree, fd);
}

/*!
 * \brief File open operation.
 */
static int fusepak_open(const char *path, struct fuse_file_info *fi)
{
	return tree_open(path, fi, tree);
}

/*!
 * \brief Read data from an open file.
 */
static int fusepak_read(const char *path, char *buf, size_t size,
			off_t offset, struct fuse_file_info *fi)
{
	// TODO: handle mutex here, not in tree_read?
	return tree_read(path, buf, size, offset, fi, tree, fd, &mutex);
}

/*!
 * Open directory.
 */
static int fusepak_opendir(const char *path, struct fuse_file_info *fi)
{
	return tree_opendir(path, fi, tree);
}

/*!
 * \brief Read directory.
 */
static int fusepak_readdir(const char *path, void *buf,
			   fuse_fill_dir_t filler, off_t offset,
			   struct fuse_file_info *fi)
{
	return tree_readdir(path, buf, filler, offset, fi, tree);
}

/*!
 * \brief Clean up filesystem.
 */
static void fusepak_destroy(void *unused)
{
	close(fd);
	tree_free(tree);
}

/*!
 * \brief The FUSE file system operations.
 */
static struct fuse_operations fusepak_operations = {
	.getattr = fusepak_getattr,
	.open = fusepak_open,
	.read = fusepak_read,
	.opendir = fusepak_opendir,
	.readdir = fusepak_readdir,
	.destroy = fusepak_destroy,
};

/*!
 * \brief Number of supported formats.
 */
static const int format_number =
    sizeof(fusepak_formats) / sizeof(struct fusepak_format *);

int fusepak_detect(const char *filename, struct fusepak_format **format)
{
	int i;

	fd = open(filename, O_RDONLY);
	if (fd < 0)
		return -1;

	for (i = 0; i < format_number; i++) {
		lseek(fd, 0, SEEK_SET);
		tree = fusepak_formats[i]->init(fd);
		if (tree) {
			*format = fusepak_formats[i];
			lseek(fd, 0, SEEK_SET);
			return 0;
		}
	}

	close(fd);
	return 1;
}

int fusepak_main(int argc, char *argv[])
{
	return fuse_main(argc, argv, &fusepak_operations, NULL);
}

char *read_string(int fd)
{
	static char buffer[1024];
	char *end;
	int ret;

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return NULL;

	end = memchr(buffer, 0, ret);
	if (end) {
		lseek(fd, -(ret - (end - buffer + 1)), SEEK_CUR);
		return buffer;
	} else
		return NULL;
}
