/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file fusepak.h
 * \author Janusz Dziemidowicz
 * \brief Framework for adding new formats.
 */

#ifndef _FUSEPAK_H_
#define _FUSEPAK_H_

//! FUSE library compliance level.
#define FUSE_USE_VERSION 26
#include <fuse.h>

#include "pack.h"
#include "wad.h"
#include "wad2.h"
#include "lgp.h"
#include "dune2.h"
#include "ksgrp.h"

/*!
 * \brief Structure describing one format supported by fusepak.
 *
 * This structure contains all information needed by fusepak to
 * support one format (for example PACK). To extend fusepak by another
 * format, one should provide such structure with all fields
 * filled. Please refer field descriptions for details.
 */
struct fusepak_format {
	/*!
	 * \brief Name of the format.
	 *
	 * This field should contain friendly name that identifies
	 * file format (examples are: PACK or WAD).
	 */
	char *format_name;

	/*!
	 * \brief Initialization/detection function.
	 *
	 * This function, when called, should attemt to prase given
	 * file, store all files in \c tree structure and return it.
	 * \param fd descriptor of file to parse.
	 * \return NULL if the given file cannot be parsed, filled \c
	 * tree structure otherwise.
	 */
	struct tree *(*init) (const int fd);
};

/*!
 * \brief Detect format of given file.
 *
 * This function runs \c init() from every known file format to find
 * what kind of format it is. It sets various internal variables, so
 * that \c fusepak_main() can be called if this function succeds.
 * \param filename path to file, which needs to be tested.
 * \param format it will be set to matching \c fusepak_format if \c
 * init() does not return NULL.
 * \return 1 if no matching format was found, 0 if matching format was
 * found (\c format will be set).
 */
int fusepak_detect(const char *filename, struct fusepak_format **format);

/*!
 * \brief Launch FUSE filesystem.
 *
 * Mounts FUSE filesystem according to options given in \c
 * argv. Before calling this, you should call \c fusepak_detect() to
 * set all needed variables.
 * \param argc argument count.
 * \param argv argument vector.
 * \return value of \c fuse_main().
 */
int fusepak_main(int argc, char *argv[]);

/*!
 * \brief Try to read zero terminated string from file.
 *
 * Tries to read one zero terminated string from the given file. File
 * is positioned right after terminating zero. Data is read into
 * static buffer with fixed maximum length (1024). Please remember
 * that subsequent calls will change the contents of the static
 * array. You should copy what this functions returns if you plan to
 * use it later.
 * \param fd of file to read from.
 * \return NULL if it is unable to read (or the string does not fit
 * into static array), pointer to a static array with zero terminated
 * string otherwise.
 */
char *read_string(int fd);

//! Treat given memory address as a 16-bit big-endian integer.
#define BE_16(x)  ((((uint8_t*)(x))[0] << 8) | ((uint8_t*)(x))[1])

//! Treat given memory address as a 32-bit big-endian integer.
#define BE_32(x)  ((((uint8_t*)(x))[0] << 24) | \
                   (((uint8_t*)(x))[1] << 16) | \
                   (((uint8_t*)(x))[2] << 8) | \
                    ((uint8_t*)(x))[3])

//! Treat given memory address as a 16-bit little-endian integer.
#define LE_16(x)  ((((uint8_t*)(x))[1] << 8) | ((uint8_t*)(x))[0])

//! Treat given memory address as a 32-bit little-endian integer.
#define LE_32(x)  ((((uint8_t*)(x))[3] << 24) | \
                   (((uint8_t*)(x))[2] << 16) | \
                   (((uint8_t*)(x))[1] << 8) | \
                    ((uint8_t*)(x))[0])

#endif				// _FUSEPAK_H_
