/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file ksgrp.c
 * \author Philipp Kutin
 * \brief KenSilverman GRP (Duke3D etc.) file support.
 */

#include "ksgrp.h"

/*!
 * \brief Read \c ksgrp_header from file.
 *
 * Tries to read one \c ksgrp_header from given file.
 * \param fd descriptor of file to read from.
 * \param header pointer to \c ksgrp_header structure which should be
 * filled.
 * \return 0 if \c header was read successfully, -1 on errors, 1 if
 * it was unable to read header.
 */
static int read_ksgrp_header(int fd, struct ksgrp_header *header)
{
	int ret;
	char buffer[16];

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	memcpy(header->magic, buffer, 12);
	header->dir_size = LE_32(&buffer[12]);
	header->dir_offset = 16;

	return 0;
}

/*!
 * \brief Read \c ksgrp_entry from file.
 *
 * Tries to read one \c ksgrp_entry from given file.
 * \param fd descriptor of file to read from.
 * \param entry pointer to \c ksgrp_entry structure which should be
 * filled.
 * \param offset offset at which data for current entry starts.
 * \return 0 if \c one entry was read successfully, -1 on errors, 1 if
 * it was unable to read one entry.
 */
static int read_ksgrp_entry(int fd, struct ksgrp_entry *entry, uint32_t offset)
{
	int ret;
	char buffer[16];

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	entry->size = LE_32(&buffer[12]);
	entry->offset = offset;
	memcpy(entry->filename, buffer, 12);
	entry->filename[12] = 0;

	return 0;
}

// **********************************************************************
// fusepak operations
// **********************************************************************

/*!
 * \brief Initialization/detection function.
 */
static struct tree *ksgrp_init(const int fd)
{
	uint32_t i;
	struct ksgrp_entry entry;
	struct ksgrp_header header;
	off_t size;
	struct tree *tree;

	uint32_t offset;

	// Read the header from KSGRP file.
	if (read_ksgrp_header(fd, &header)) {
		fprintf(stderr, "unable to read header from KenSilverman GRP file\n");
		return NULL;
	}

	if (strncmp(header.magic, "KenSilverman", 12)) {
		fprintf(stderr, "not a KenSilverman GRP file\n");
		return NULL;
	}

	size = lseek(fd, 0, SEEK_END);

	tree = tree_empty();

	// Read all entries from KSGRP file. Also perform some simple
	// sanity tests on read values.
	lseek(fd, header.dir_offset, SEEK_SET);
	offset = 16 + header.dir_size*16;

	for (i = 0; i < header.dir_size; i++) {
		if (read_ksgrp_entry(fd, &entry, offset)) {
			fprintf(stderr, "unable to read entry from group file\n");
			tree_free(tree);
			return NULL;
		}

		if (entry.offset + entry.size > size) {
			fprintf(stderr, "damaged group file\n");
			tree_free(tree);
			return NULL;
		}
		tree_insert(tree, entry.filename,
			    strlen(entry.filename), entry.offset,
			    entry.size);

		offset += entry.size;
	}

	return tree;
}

struct fusepak_format fusepak_ksgrp = {
	.format_name = "KSGRP",
	.init = ksgrp_init,
};
