/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file ksgrp.h
 * \author Philipp Kutin
 * \brief KenSilverman GRP (Duke3D etc.) file support header file.
 */

#ifndef _KSGRP_H_
#define _KSGRP_H_

#define _GNU_SOURCE

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "tree.h"
#include "fusepak.h"

/*!
 * \brief Header of the KSGRP file.
 *
 * This structures describes the header that can be found on the
 * beginning of every KSGRP file. KSGRP files are simple archives (no
 * compression at all) used by many games using idSoftware's engine
 * (Doom, Doom 2).
 */
struct ksgrp_header {
	/*!
	 * \brief KSGRP file identification.
	 *
	 * Legitimate KSGRP file should contain string "KenSilverman" here.
	 */
	char magic[12];

	/*!
	 * \brief Number of entries.
	 *
	 * This is the number of \c ksgrp_entry structures under \c
	 * dir_offset offset in the KSGRP file.
	 */
	uint32_t dir_size;

	/*!
	 * \brief Position of directory from start of file
	 *
	 * This is the offset from beginning of the KSGRP file. Under
	 * this offset there will be \a dir_size \c ksgrp_entry
	 * structures.
	 */
	uint32_t dir_offset;
};

/*!
 * \brief One entry (file) inside of KSGRP file.
 *
 * This structures describes one entry inside of KSGRP file. In fact
 * this entry should be considered as a file (although it is also
 * called a lump).
 */
struct ksgrp_entry {
	/*!
	 * \brief Offset of file inside of KSGRP file.
	 *
	 * This field defines at what offset the file described by \c
	 * ksgrp_entry can be found inside of KSGRP file.
	 */
	uint32_t offset;

	/*!
	 * \brief Size of file.
	 *
	 * This field describes size of the file described in \c
	 * ksgrp_entry inside of KSGRP file.
	 */
	uint32_t size;

	/*!
	 * \brief Filename.

	 * Name of the file. Padded with zeroes.  Maximum filename string
	 * length is 8+1+3  (DOS filename). There is no directory structure (implicit or
	 * explicit) in KSGRP files.
	 */
	char filename[13];
};

/*!
 * \brief Structure \c fusepak_format describing KSGRP format.
 */
extern struct fusepak_format fusepak_ksgrp;

#endif				// _KSGRP_H_
