/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file lgp.c
 * \author Janusz Dziemidowicz
 * \brief LGP file support.
 */

#include "lgp.h"

/*!
 * \brief Read \c lgp_header from file.
 *
 * Tries to read one \c lgp_header from given file.
 * \param fd descriptor of file to read from.
 * \param header pointer to \c lgp_header structure which should be
 * filled.
 * \return 0 if \c header was read successfully, -1 on errors, 1 if
 * it was unable to read header.
 */
static int read_lgp_header(int fd, struct lgp_header *header)
{
	int ret;
	char buffer[16];

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	header->null[0] = buffer[0];
	header->null[1] = buffer[1];
	memcpy(header->magic, &buffer[2], 10);
	header->dir_size = LE_32(&buffer[12]);

	return 0;
}

/*!
 * \brief Read \c lgp_entry from file.
 *
 * Tries to read one \c lgp_entry from given file.
 * \param fd descriptor of file to read from.
 * \param entry pointer to \c lgp_entry structure which should be
 * filled.
 * \return 0 if \c one entry was read successfully, -1 on errors, 1 if
 * it was unable to read one entry.
 */
static int read_lgp_entry(int fd, struct lgp_entry *entry)
{
	int ret;
	char buffer[27];

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	memcpy(entry->filename, buffer, 20);
	entry->offset = LE_32(&buffer[20]);
	entry->unknown[0] = buffer[24];
	entry->unknown[1] = buffer[25];
	entry->unknown[2] = buffer[26];

	return 0;
}

/*!
 * \brief Read \c lgp_file_header from file.
 *
 * Tries to read one \c lgp_file_header from given file.
 * \param fd descriptor of file to read from.
 * \param header pointer to \c lgp_file_header structure which should
 * be filled.
 * \return 0 if \c one header was read successfully, -1 on errors, 1 if
 * it was unable to read one header.
 */
static int read_lgp_file_header(int fd, struct lgp_file_header *header)
{
	int ret;
	char buffer[24];

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	memcpy(header->filename, buffer, 20);
	header->size = LE_32(&buffer[20]);

	return 0;
}

// **********************************************************************
// fusepak operations
// **********************************************************************

/*!
 * \brief Initialization/detection function.
 */
static struct tree *lgp_init(const int fd)
{
	uint32_t i;
	struct lgp_entry entry;
	struct lgp_file_header file_header;
	struct lgp_header header;
	off_t size;
	struct tree *tree;
	char id[14];

	// Read the header from LGP file.
	if (read_lgp_header(fd, &header)) {
		fprintf(stderr, "file too small to be a LGP file\n");
		return NULL;
	}

	if (header.null[0] != 0 || header.null[1] != 0) {
		fprintf(stderr, "not a LGP file\n");
		return NULL;
	}

	if (strncmp(header.magic, "SQUARESOFT", 10)) {
		fprintf(stderr, "not a LGP file\n");
		return NULL;
	}

	lseek(fd, -14, SEEK_END);
	if (read(fd, id, sizeof(id)) < sizeof(id)) {
		fprintf(stderr, "read error\n");
		return NULL;
	}
	if (strncmp(id, "FINAL FANTASY7", sizeof(id))) {
		fprintf(stderr, "not a LGP file\n");
		return NULL;
	}

	size = lseek(fd, 0, SEEK_END);

	tree = tree_empty();

	// Read all entries from LGP file.
	for (i = 0; i < header.dir_size; i++) {
		// Go to next directory entry
		lseek(fd, sizeof(struct lgp_header) + i * 27, SEEK_SET);
		if (read_lgp_entry(fd, &entry)) {
			fprintf(stderr, "unable to read entry from LGP file\n");
			tree_free(tree);
			return NULL;
		}

		if (entry.offset > size) {
			fprintf(stderr, "damaged LGP file\n");
			tree_free(tree);
			return NULL;
		}
		// Go to file data, it starts with a header with it's size
		lseek(fd, entry.offset, SEEK_SET);
		if (read_lgp_file_header(fd, &file_header)) {
			fprintf(stderr,
				"unable to read file header from LGP file\n");
			tree_free(tree);
			return NULL;
		}

		if (entry.offset + sizeof(struct lgp_file_header) +
		    file_header.size > size) {
			fprintf(stderr, "damaged LGP file\n");
			tree_free(tree);
			return NULL;
		}
		tree_insert(tree, entry.filename, strlen(entry.filename),
			    entry.offset + sizeof(struct lgp_file_header),
			    file_header.size);
	}

	return tree;
}

struct fusepak_format fusepak_lgp = {
	.format_name = "LGP",
	.init = lgp_init,
};
