/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file lgp.h
 * \author Janusz Dziemidowicz
 * \brief LGP file support header file.
 */

#ifndef _LGP_H_
#define _LGP_H_

#define _GNU_SOURCE

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "tree.h"
#include "fusepak.h"

/*!
 * \brief Header of the LGP file.
 *
 * This structures describes the header that can be found on the
 * beginning of every LGP file. LGP files are simple archives (no
 * compression at all) used by PC version of Final Fantasy VII.
 */
struct lgp_header {
	/*!
	 * \brief Two null bytes.
	 *
	 * Every LGP file begins with two null bytes.
	 */
	uint8_t null[2];

	/*!
	 * \brief LGP file identification.
	 *
	 * Legitimate LGP file should contain string "SQUARESOFT"
	 * here.
	 */
	char magic[10];

	/*!
	 * \brief Number of entries.
	 *
	 * This is the number of \c lgp_entry structures after this
	 * header in the LGP file.
	 */
	uint32_t dir_size;
};

/*!
 * \brief One entry (file) inside of LGP file.
 *
 * This structures describes one entry inside of LGP file. In fact
 * this entry should be considered as a file. Correct number of this
 * structures is right after the header of LGP file.
 */
struct lgp_entry {
	/*!
	 * \brief Filename.
	 *
	 * Null terminated filename.
	 */
	char filename[20];

	/*!
	 * \brief Offset of file inside of LGP file.
	 *
	 * This field defines at what offset the file described by \c
	 * lgp_entry can be found inside of LGP file. Please note that
	 * each file starts with \c lgp_file_header structure.
	 */
	uint32_t offset;

	/*!
	 * \brief Unknown.
	 *
	 * Unknown three bytes. Seems to be some kinf of check code.
	 */
	uint8_t unknown[3];
};

/*!
 * \brief Header of one file inside of the LGP file.
 *
 * Each file data (as pointed by the \c offset field from \c lgp_entry
 * structure) starts with this structure. Right after it sits the file
 * data.
 */
struct lgp_file_header {
	/*!
	 * \brief Filename.
	 *
	 * Filename one more time. Game engine seems to ignore this
	 * anyway and uses the filename from \c lgp_entry. Null terminated.
	 */
	char filename[20];

	/*!
	 * \brief Size of file.
	 *
	 * Size of the file data that comes right after this structure.
	 */
	uint32_t size;
};

/*!
 * \brief Structure \c fusepak_format describing LGP format.
 */
extern struct fusepak_format fusepak_lgp;

#endif				// _LGP_H_
