/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file pack.h
 * \author Janusz Dziemidowicz
 * \brief PACK file support header file.
 */

#ifndef _PACK_H_
#define _PACK_H_

#define _GNU_SOURCE

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "tree.h"
#include "fusepak.h"

/*!
 * \brief Header of the PACK file.
 *
 * This structures describes the header that can be found on the
 * beginning of every PACK file. PACK files are simple archives (no
 * compression at all) used by many games using idSoftware's engine
 * (Quake, Quake 2, Half-Life).
 */
struct pack_header {
	/*!
	 * \brief PACK file identification.
	 *
	 * Legitimate PACK file should contain string "PACK" here.
	 */
	char magic[4];

	/*!
	 * \brief Position of directory from start of file
	 *
	 * This is the offset from beginning of the PACK file. Under
	 * this offset there will be \a dir_size /0x40 \c pack_entry
	 * structures.
	 */
	uint32_t dir_offset;

	/*!
	 * \brief Number of entries.
	 *
	 * This is the number of \c pack_entry structures under \c
	 * dir_offset offset in the PACK file. This number is
	 * multiplied by 0x40, so in order to get number of entries,
	 * one should get \a dir_size /0x40.
	 */
	uint32_t dir_size;
};

/*!
 * \brief One entry (file) inside of PACK file.
 *
 * This structures describes one entry inside of PACK file. In fact
 * this entry should be considered as a file.
 */
struct pack_entry {
	/*!
	 * \brief Filename.

	 * Name of the file. Since there is no directory structure in
	 * PACK file, filenames in entries can contain '/'. This means
	 * that the directory structure has to be guessed based on
	 * filenames.  Padded with zeroes. Maximum filename length is
	 * 56.
	 */
	char filename[56];

	/*!
	 * \brief Offset of file inside of PACK file.
	 *
	 * This field defines at what offset the file described by \c
	 * pack_entry can be found inside of PACK file.
	 */
	uint32_t offset;

	/*!
	 * \brief Size of file.
	 *
	 * This field describes size of the file described in \c
	 * pack_entry inside of PACK file.
	 */
	uint32_t size;
};

/*!
 * \brief Structure \c fusepak_format describing PACK format.
 */
extern struct fusepak_format fusepak_pack;

#endif				// _PACK_H_
