/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file wad.c
 * \author Janusz Dziemidowicz
 * \brief WAD file support.
 */

#include "wad.h"

/*!
 * \brief Read \c wad_header from file.
 *
 * Tries to read one \c wad_header from given file.
 * \param fd descriptor of file to read from.
 * \param header pointer to \c wad_header structure which should be
 * filled.
 * \return 0 if \c header was read successfully, -1 on errors, 1 if
 * it was unable to read header.
 */
static int read_wad_header(int fd, struct wad_header *header)
{
	int ret;
	char buffer[12];

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	memcpy(header->magic, buffer, 4);
	header->dir_size = LE_32(&buffer[4]);
	header->dir_offset = LE_32(&buffer[8]);

	return 0;
}

/*!
 * \brief Read \c wad_entry from file.
 *
 * Tries to read one \c wad_entry from given file.
 * \param fd descriptor of file to read from.
 * \param entry pointer to \c wad_entry structure which should be
 * filled.
 * \return 0 if \c one entry was read successfully, -1 on errors, 1 if
 * it was unable to read one entry.
 */
static int read_wad_entry(int fd, struct wad_entry *entry)
{
	int ret;
	char buffer[16];

	ret = read(fd, buffer, sizeof(buffer));
	if (ret < 0)
		return -1;
	if (ret < sizeof(buffer))
		return 1;

	entry->offset = LE_32(buffer);
	entry->size = LE_32(&buffer[4]);
	memcpy(entry->filename, &buffer[8], 8);

	return 0;
}

// **********************************************************************
// fusepak operations
// **********************************************************************

/*!
 * \brief Initialization/detection function.
 */
static struct tree *wad_init(const int fd)
{
	uint32_t i;
	struct wad_entry entry;
	struct wad_header header;
	off_t size;
	struct tree *tree;

	// Read the header from WAD file.
	if (read_wad_header(fd, &header)) {
		fprintf(stderr, "unable to read header from WAD file\n");
		return NULL;
	}

	if (strncmp(header.magic, "IWAD", 4)
	    && strncmp(header.magic, "PWAD", 4)) {
		fprintf(stderr, "not a wad file\n");
		return NULL;
	}

	size = lseek(fd, 0, SEEK_END);

	tree = tree_empty();

	// Read all entries from WAD file. Also perform some simple
	// sanity tests on read values.
	lseek(fd, header.dir_offset, SEEK_SET);
	for (i = 0; i < header.dir_size; i++) {
		if (read_wad_entry(fd, &entry)) {
			fprintf(stderr, "unable to read entry from wad\n");
			tree_free(tree);
			return NULL;
		}

		if (entry.offset + entry.size > size) {
			fprintf(stderr, "damaged wad file\n");
			tree_free(tree);
			return NULL;
		}
		tree_insert(tree, entry.filename,
			    strnlen(entry.filename, 8), entry.offset,
			    entry.size);
	}

	return tree;
}

struct fusepak_format fusepak_wad = {
	.format_name = "WAD",
	.init = wad_init,
};
