/*
 *  Copyright (C) 2005 Janusz Dziemidowicz (rraptorr@nails.eu.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * \file wad2.h
 * \author Janusz Dziemidowicz
 * \brief WAD2 file support header file.
 */

#ifndef _WAD2_H_
#define _WAD2_H_

#define _GNU_SOURCE

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "fusepak.h"

/*!
 * \brief Header of the WAD2 file.
 *
 * This structures describes the header that can be found on the
 * beginning of every WAD2 file. WAD2 files are simple archives (no
 * compression at all) used by many games using idSoftware's engine
 * (Quake).
 */
struct wad2_header {
	/*!
	 * \brief WAD2 file identification.
	 *
	 * Legitimate WAD2 file should contain string "WAD2" here.
	 */
	char magic[4];

	/*!
	 * \brief Number of entries.
	 *
	 * This is the number of \c wad2_entry structures under \c
	 * dir_offset offset in the WAD2 file.
	 */
	uint32_t dir_size;

	/*!
	 * \brief Position of directory from start of file
	 *
	 * This is the offset from beginning of the WAD2 file. Under
	 * this offset there will be \a dir_size \c wad2_entry
	 * structures.
	 */
	uint32_t dir_offset;
};

/*!
 * \brief One entry (file) inside of WAD2 file.
 *
 * This structures describes one entry inside of WAD2 file. In fact
 * this entry should be considered as a file (although it is also
 * called a lump).
 */
struct wad2_entry {
	/*!
	 * \brief Offset of file inside of WAD2 file.
	 *
	 * This field defines at what offset the file described by \c
	 * wad2_entry can be found inside of WAD2 file.
	 */
	uint32_t offset;

	/*!
	 * \brief Size of file in WAD2 file.
	 *
	 * This field describes size of the file described in \c
	 * wad2_entry inside of WAD2 file.
	 */
	uint32_t dsize;

	/*!
	 * \brief Size of file in memory.
	 */
	uint32_t size;

	/*!
	 * \brief Type of file.
	 */
	char type;

	/*!
	 * \brief Compression type.
	 */
	char compression;

	/*!
	 * \brief Unused.
	 */
	uint16_t dummy;

	/*!
	 * \brief Filename.

	 * Name of the file. Padded with zeroes.  Maximum filename
	 * length is 16. There is no directory structure (implicit or
	 * explicit) in WAD2 files.
	 */
	char filename[16];
};

/*!
 * \brief Structure \c fusepak_format describing WAD2 format.
 */
extern struct fusepak_format fusepak_wad2;

#endif				// _WAD2_H_
